clear, clc, close all

% ------------------------ parameters section -------------------------------
interactive = 1;

% ------------------------ here be the code ---------------------------------
fprintf([
        'Welcome to the clustering evaluation demo - part 1\n\n', ...
        'In this experiment we will try different evaluation techniques, with the aim\n', ...
        'of learning something more about a dataset.\n\n'
        ]);

if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

fprintf([
        'First of all, let us generate a test dataset.\n', ...
        'The following lines of code will call the "genDataset" function, which\n', ...
        'generates 5 globular clusters of 20 points each, and the "plotClustering"\n',...
        'function to plot them.\n\n'
        ]);

if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

% if you want to do different experiments on the same dataset, generate
% it once, save it, and reload it here instead of calling genDataset
clusters = 5;
numPts = 200;
ptsClust = numPts/clusters;
minRadius = 10; maxRadius = 20;
maxX = 100; maxY = 100;

% check the meaning of genDataSet parameters in the function's code
[coords,classes]=genDataSet(1,clusters,ptsClust,ptsClust,minRadius,maxRadius,maxX,maxY,0);
figure;plotClustering(coords,classes,classes);
title('Data plot (colors/shapes represent different classes)')
drawnow();

fprintf('Dataset characteristics: %d points, %d-dimensional space, %d clusters\n\n',size(coords,1),size(coords,2),clusters);

fprintf([
        'Note that, despite the fact you chose k distinct clusters, some of them\n',...
        'might overlap. No problem, that will just make everything more realistic\n',...
        '(i.e. in reality you will very likely have overlapping sets).\n\n',...
        'Now suppose you are given these data without knowing much about them.\n',...
        'The first thing you might want to do is verify whether they have any\n',...
        '/clustering tendency/, i.e. they are more likely to be clustered than random\n',...
        'data. To do this, let us apply a statistical approach: we will use some metric\n',...
        'on, let''s say, 1000 randomly generated datasets and compare its results with\n',...
        'the one we get on our data. The more statistically unlikely our result, the\n',...
        'higher the probability that our data can be clustered.\n',...
        'QUESTION: what does the "clusteringTendency" function do? (hint: check the code)\n\n'
        ]);

if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

% NOTE: you can pass the number of random datasets as a second parameter.
% The higher it is, the more accurate the histogram you will get
randpoints = clusteringTendency(coords);

fprintf([
        'As you can see, the same metric applied on our dataset yields a value which is\n', ...
        'very different from the ones we get in the random case. Now let us try another\n', ...
        'test: as we know points are already sorted by cluster (see the "classes" variable),\n', ...
        'let us see their distances: what we expect to see are smaller (blue) distances\n',...
        'for the points in the same cluster, bigger (red) distances for the ones in another.\n',...
        'As the points are sorted by cluster, we also expect to see big blocks of close\n',...
        'points together on the matrix diagonal.\n\n'
        ]);

if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

d1 = L2_distance(coords',coords');
%d2 = L2_distance(randpoints',randpoints');
figure; 
%subplot(1,2,1);
imagesc(d1), axis image
title('Distance matrix (generated dataset)');
%subplot(1,2,2);
%imagesc(d2), axis image
%title('Distance matrix (random dataset)');
drawnow()

fprintf([
        'Note that if two or more clusters are overlapping you will see bigger blocks,\n',...
        'or blue blocks far from the diagonal. In these cases, automatically detecting\n',...
        'the best (i.e. correct, according to ground truth) number of clusters will be\n',...
        'more complex. Let us use the "elbow method" to verify how well we can predict\n',...
        'the number of clusters in the dataset.\n'
        ]);

if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

% write the code to plot SSE wrt k here

fprintf([
        'You will need to write the code to calculate SSE and plot the elbow: you can\n',...
        'find all the information you need in the previous evaluation tutorial:\n\n',...
        'http://davide.eynard.it/2012/06/05/octave-clustering-demo-part-2-cluster-evaluation/ \n\n',...
        '(HINT: use the functions provided in this bundle, as they fix some previous bugs)\n',...
        'After you verify which is the most likely value for k, use it in the next part\n',...
        'of the code to run k-means on the synthetic blobs dataset.\n\n'
        ]);

if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

%%
k = 5;
labels = repKmeans(coords,k,100);
a = uAveragedAccuracy(labels,classes);
fprintf('K-Means was able to detect clusters with an accuracy of %.2f%%\n\n',a*100);

figure;plotClustering(coords,labels,classes);
title(sprintf('K-Means clustering (colors/shapes represent different labels/classes). Accuracy=%.2f%%',a*100))
drawnow();

figure;imagesc(repmat(labels,1,2));
title(sprintf('K-Means clustering (label values are directly plotted). Accuracy=%.2f%%',a*100))
drawnow();

fprintf([
        'So, we /know/ (as we have generated it!) that our synthetic dataset has\n',...
        '5 clusters. Looking for the elbow might help us find the correct k or not,\n',...
        'according to how well separated the clusters are.\n',...
        'To inspect clustering results, we have plotted our dataset using the\n',...
        '"plotClustering" function (point colors match labels, point shapes match\n',...
        'classes, i.e. ground truth). This can be done only if data is 2- or \n',...
        '3-dimensional. For higher dimensionality, a valid alternative is to plot only\n',...
        'the label values: if points are sorted w.r.t. the clusters they belong to, then\n',...
        'we can expect to see large colored stripes in the figure. Data which has been\n',...
        'wrongly clustered appears with different colors instead.\n\n',...
        'QUESTION: is there any difference in accuracy if we choose k=5 (the default) or a\n',...
        'different value suggested by the elbow method?\n',...
        '(HINT: if the method suggests 5, run again until you have a "bad" dataset)\n\n'
        ]);
