clear, clc, close all

% ------------------------ parameters section -------------------------------
interactive = 1;

% ------------------------ here be the code ---------------------------------
load datasets

fprintf([
        'Welcome to the clustering evaluation demo - part 2\n\n', ...
        'Now we will try to use what we learned to study an unknown dataset.\n',...
        'Of the three datasets in memory (data1, data2, and data3) only one contains\n',...
        'interesting data. Find which dataset is the interesting one, by verifying its\n',...
        'clustering tendency and peeking at its contents.\n\n'
        ]);

if (interactive) fprintf('Press a key to continue (and then wait... this might take a while). '); pause; end 

% assign one of the three datasets (data1, data2, or data3) to the variable 
% coords and run the clusteringTendency function

% coords = data1; % or data2 or data3

if ~exist('coords','var')
    return
end

clusteringTendency(coords,1000);

fprintf([
        'Ok, if you have checked all the datasets you can clearly see that there\n',...
        'is definitely one which has clustering tendency and contains non-trivial\n',... 
        'data. Can we also get an idea of how many clusters it contains?\n',...
        'As a first experiment, try to plot k-Means SSE for increasing values of k\n', ...
        'and verify whether the plot shows an elbow.\n\n';
        ]);

return % comment this to continue the experiment
if (interactive) fprintf('Press a key to continue (and then wait... this might take a while).\n\n'); pause; end

tic
for i = 5:15; [labels(:,i),centroids,sse(i)]=repKmeans(coords,i,50);end
toc
figure;plot(5:15,sse(5:15));
drawnow


fprintf([
        'The SSE plot might not give you meaningful results (if you run it many\n',...
        'times, you might see different curves - try increasing k-Means replicates).\n',...
        'Let us try to plot the distance matrix instead:\n\n', ...
        ]);

return % comment this to continue the experiment
if (interactive) fprintf('Press a key to continue.\n\n'); pause; end


% add the code to plot the distance matrix here

fprintf([
        'Good! Looking at the distance matrix we can see that data is not only non\n',...
        'random, but also already sorted into clusters (blue blocks on the diagonal).\n',...
        'Also, most of the blocks are size 10: what if we just have 10x10 (possibly\n',...
        'slightly overlapping) clusters? If we do, then we can reconstruct the ground\n',...
        'truth and use an external measure (e.g. accuracy) to evaluate how good\n',...
        'our clustering is.\n\n', ...
        ]);

if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

k = 10;

% manually rebuild ground truth: create a "classes" vector where the first
% 10 elements have value 1, the next 10 have value 2, and so on until the
% last 10 elements with value 10. The vector will have length 100.

% classes = ...

if ~exists('classes','var')
    return
end

labels = repKmeans(coords,k,100);
a = uAveragedAccuracy(labels,classes);
figure;imagesc(repmat(labels,1,2));
title(sprintf('K-Means clustering (label values are directly plotted). Accuracy=%.2f%%',a*100))
drawnow();

fprintf([
        'k-Means has accuracy a=%.2f%%. As the dataset is 4096-dimensional, to get a\n',...
        'visual cue about the algorithm performance we directly plot the labels.\n', ...
        'Dimensionality in this experiment is a crucial issue: k-Means does not scale\n',...
        'well with the number of dimensions (you have experienced how slower it became,\n',...
        'even with just 100 points!), plus it considers all of them equally, no matter\n',...
        'if data lies on a manifold with a lower dimensionality.\n\n',...
        'Is there a way to reduce the dimensionality of the problem and address some\n',...
        'of the limitations of k-Means? Let us try spectral clustering!\n\n',...
        ], a*100);

return % comment this to continue the experiment
if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

nn = 10;
t = 0;
W = adjacency_weighted(coords,nn,t);

[L,invSqD] = laplacian(W);

% get eigenvalues 
[V,E] = eigs(L,10,'SA'); 

% normalize and sort V according to the (ascending) order of eigenvalues
V = invSqD * V;
[~,i] = sort(diag(E));
V = V(:,i);

labels = repKmeans(V(:,1:10),10,200);
a = uAveragedAccuracy(labels,classes);
figure;imagesc(repmat(labels,1,2));
title(sprintf('Spectral clustering (label values are directly plotted). Accuracy=%.2f%%',a*100))
drawnow();

fprintf([
        'Spectral clustering has a better accuracy: a=%.2f%%. Feel free to play with\n',...
        'the parameters used to build the adjacency matrix W and see if you can further\n',...
        'increase it.\n\n',...
        'The only question now is: what have we clustered? Well, here is the result of\n',...
        'our work:\n\n',...
        ], a*100);
    
return % comment this to continue the experiment
if (interactive) fprintf('Press a key to continue.\n\n'); pause; end

cm = colormap('jet');
idx = reshape([1:100],[10 10])';
yyy = [];
for i = 1 : 10
    xxx = []; 
    for j = 1:10
        k =idx(i,j); 
        xxx=cat(2,xxx,[cat(1,repmat(reshape(coords(k,:),[64 64]),[1 1 3])/255,repmat(reshape(cm(6*labels(k),:),[1 1 3]),4,64)) zeros(68,1,3)]); 
    end
    yyy=cat(1,yyy,xxx);
end
figure;imagesc(yyy),axis off image

fprintf([
        'TA-DAAAH! :-)\n\n',...
        'All those 4096-sized data points were actually 64x64 images from the "Olivetti\n',...
        'faces" dataset (find a copy at http://www.cs.nyu.edu/~roweis/data.html). Below\n',...
        'each image, you will find a colored stripe showing the label it has been assigned.\n',...
        'As you can see, our clustering algorithm has done a pretty good job, assigning\n',...
        'very few images to the wrong clusters.\n\n'
        ]);