function [L_sym,invSqD] = laplacian(W)
% given a weighted adjacency matrix W, returns the symmetric
% normalized Laplacian and D^(-0.5), needed to normalize the
% Laplacian's eigenspace once it is calculated

	D = diag(sum(W));
	% build the "standard", unnormalized laplacian
	L = D - W;

	% build the normalized, symmetric laplacian L_sym
	invSqD=sparse(full(D)^(-1/2));
	L_sym = invSqD * L * invSqD;
    
    % make sure L_sym is symmetric ( not just numerically...)
    L_sym = (L_sym+L_sym')/2;
end
