function[centroid, pointsInCluster, assignment]= myKmeans(data, nbCluster)
% usage
% function[centroid, pointsInCluster, assignment]=
% myKmeans(data, nbCluster)
%
% Output:
% centroid: matrix in each row are the Coordinates of a centroid
% pointsInCluster: row vector with the nbDatapoints belonging to
% the centroid
% assignment: row Vector with clusterAssignment of the dataRows
%
% Input:
% data in rows
% nbCluster : nb of centroids to determine
%
% (c) by Christian Herta ( www.christianherta.de )
% Modified by Nghia Ho to improve speed
% Further improved/debugged by Davide Eynard (davide.eynard.it) in Dec 2013

data_dim = length(data(1,:));

% choose centroids from the actual data points
idx_ = randperm(size(data,1));
idx_ = idx_(1:nbCluster);
centroid = data(idx_,:);

% no stopping at start
pos_diff = 1.;
 
% main loop until
while pos_diff > 0.0
  % E-Step
  % assign each datapoint to the closest centroid 
  if(nbCluster == 1) % special case
	assignment = ones(size(data,1), 1);
  else
	  [~, assignment] = min(L2_distance(centroid',data'));
	  assignment = assignment';
  end
 
  % for the stoppingCriterion
  oldPositions = centroid;
 
  % M-Step
  % recalculate the positions of the centroids
  centroid = zeros(nbCluster, data_dim);
  pointsInCluster = zeros(nbCluster, 1);
 
  for c = 1: nbCluster
	d = data(assignment == c,:);
	pointsInCluster(c,1) = size(d,1);
 
    if( pointsInCluster(c, 1) ~= 0)
        centroid(c,:) = sum(d);
        centroid( c , : ) = centroid( c, : ) / pointsInCluster(c, 1);
    else
        % set cluster to a new random data point (which is not centroid)
        ccc=setdiff(data,centroid,'rows');
        centroid(c,:)=ccc(randi(size(ccc,1)),:);
    end
  end
 
  %stoppingCriterion
  pos_diff = sum (sum( (centroid - oldPositions).^2 ) );
end
end